#ifndef SWIG_LIBWCCL_OPERATOR_I
#define SWIG_LIBWCCL_OPERATOR_I

%module libccloperator
%{
  #include <libwccl/ops/operator.h>
%}


%include "libcclvalue.i"
%include "libcclsentencecontext.i"
%include "libcclfunctionaloperator.i"

/*
%include "libcclvariables.i"
%include "libcclparsedexpression.i"
*/

%include "std_string.i"
%include "boost_shared_ptr.i"

%feature("notabstract") Wccl::Operator;

namespace Wccl {
  template <class T> class Operator : public FunctionalOperator {
  public:
    Operator();
    Operator(const Operator& other, bool clean = false);

    /* --------------------------------------------------------------------- */
    %rename(OperatorEq) *::operator=(const Operator& other);

    /* --------------------------------------------------------------------- */
    %rename(OpFun) operator()(const SentenceContext& sentence_context);
    boost::shared_ptr<const T> operator()(const SentenceContext& sentence_context);

    /* --------------------------------------------------------------------- */
    boost::shared_ptr<const T> apply(const SentenceContext& sentence_context);
    boost::shared_ptr<T> copy_apply(const SentenceContext& sentence_context);
    boost::shared_ptr<const Value> base_apply(const SentenceContext& sc);
    
    /* --------------------------------------------------------------------- */
    Operator clone() const;
    Operator clone_clean() const;
    
    /* --------------------------------------------------------------------- */
    boost::shared_ptr<Operator<T> > clone_ptr() const;
    boost::shared_ptr<Operator<T> > clone_clean_ptr() const;

    /* --------------------------------------------------------------------- */
    std::string to_string(const Corpus2::Tagset& tagset) const;

  protected:
    Operator* clone_internal() const;
    std::ostream& write_to(std::ostream& ostream) const;
  };

  %template (BoolOperator) Operator<Bool>;
  %template (TSetOperator) Operator<TSet>;
  %template (StrSetOperator) Operator<StrSet>;
  %template (PositionOperator) Operator<Position>;

  %template (BoolOperatorPtr) shared_ptr<Wccl::Operator<Bool> >;
  %template (TSetOperatorPtr) shared_ptr<Wccl::Operator<TSet> >;
  %template (StrSetOperatorPtr) shared_ptr<Wccl::Operator<StrSet> >;
  %template (PositionOperatorPtr) shared_ptr<Wccl::Operator<Position> >;
}

using namespace boost;
using namespace std;
using namespace Wccl;

#endif /* SWIG_LIBWCCL_OPERATOR_I */
